@@ -23,6 +23,7 @@
 void sdl_setTorque(int, float);
 void sdl_setDamper(int, float, float);
 void sdl_setSpring(int, float, float);
+void sdl_setSine(int, float, float, u32);
 void sdl_stopHaptic(int);
 #endif
 
@@ -46,6 +47,12 @@ inline static void setSpring(int port, float saturation, float speed) {
 #endif
 }
 
+inline static void setSine(int port, float power, float frequency, u32 duration_ms) {
+#ifdef USE_SDL
+	sdl_setSine(port, power, frequency, duration_ms);
+#endif
+}
+
 inline static void stopAll(int port) {
 #ifdef USE_SDL
 	sdl_stopHaptic(port);
