@@ -45,7 +45,7 @@ public:
 		float colorClampMax[4];
 		float sp_FOG_COL_RAM[4];	// Only using 3 elements but easier for std140
 		float sp_FOG_COL_VERT[4];	// same comment
-		float ditherColorMax[4];
+		float ditherDivisor[4];
 		float cp_AlphaTestValue;
 		float sp_FOG_DENSITY;
 		float shade_scale_factor;	// new for OIT
