@@ -72,7 +72,7 @@ layout (std140, set = 0, binding = 1) uniform FragmentShaderUniforms
 	vec4 colorClampMax;
 	vec4 sp_FOG_COL_RAM;
 	vec4 sp_FOG_COL_VERT;
-	vec4 ditherColorMax;
+	vec4 ditherDivisor;
 	float cp_AlphaTestValue;
 	float sp_FOG_DENSITY;
 } uniformBuffer;
@@ -294,16 +294,14 @@ void main()
 
 #if DITHERING == 1
 	float ditherTable[16] = float[](
-		 0.9375,  0.1875,  0.75,  0.,   
-		 0.4375,  0.6875,  0.25,  0.5,
-		 0.8125,  0.0625,  0.875, 0.125,
-		 0.3125,  0.5625,  0.375, 0.625	
+		5., 13.,  7., 15.,
+		9.,  1., 11.,  3.,
+		6., 14.,  4., 12.,
+		10., 2.,  8.,  0.
 	);
 	float r = ditherTable[int(mod(gl_FragCoord.y, 4.)) * 4 + int(mod(gl_FragCoord.x, 4.))];
-	// 31 for 5-bit color, 63 for 6 bits, 15 for 4 bits
-	color += r / uniformBuffer.ditherColorMax;
-	// avoid rounding
-	color = floor(color * 255.) / 255.;
+	vec4 dv = vec4(r, r, r, 1.) / uniformBuffer.ditherDivisor;
+	color = clamp(floor(color * 255. + dv) / 255., 0., 1.);
 #endif
 	gl_FragColor = color;
 }
@@ -737,6 +735,7 @@ vk::UniqueShaderModule ShaderManager::compileShader(const VertexShaderParams& pa
 	else
 	{
 		src.addConstant("pp_Gouraud", (int)params.gouraud)
+				.addConstant("pp_Texture", (int)params.texture)
 				.addSource(GouraudSource)
 				.addSource(N2LightShaderSource)
 				.addSource(N2VertexShaderSource);
