@@ -157,20 +157,6 @@ struct retro_core_option_v2_definition option_defs_us[] = {
       },
       "disabled",
    },
-   {
-      CORE_OPTION_NAME "_boot_to_bios",
-      "Boot to BIOS (Restart Required)",
-      NULL,
-      "Boot directly into the Dreamcast BIOS menu.",
-      NULL,
-      "system",
-      {
-         { "disabled", NULL },
-         { "enabled",  NULL },
-         { NULL, NULL },
-      },
-      "disabled",
-   },
    {
       CORE_OPTION_NAME "_enable_dsp",
       "Enable DSP",
@@ -790,6 +776,20 @@ struct retro_core_option_v2_definition option_defs_us[] = {
       },
       "disabled",
    },
+   {
+      CORE_OPTION_NAME "_preload_custom_textures",
+      "Preload Custom Textures",
+      NULL,
+      "Preload custom textures at game start. May improve performance but increases memory usage.",
+      NULL,
+      "hacks",
+      {
+         { "disabled", NULL },
+         { "enabled",  NULL },
+         { NULL, NULL },
+      },
+      "disabled",
+   },
    {
       CORE_OPTION_NAME "_dump_textures",
       "Dump Textures",
@@ -804,6 +804,20 @@ struct retro_core_option_v2_definition option_defs_us[] = {
       },
       "disabled",
    },
+   {
+      CORE_OPTION_NAME "_dump_replaced_textures",
+      "Dump Replaced Textures",
+      NULL,
+      "Always dump textures that are already replaced by custom textures.",
+      NULL,
+      "hacks",
+      {
+         { "disabled", NULL },
+         { "enabled",  NULL },
+         { NULL, NULL },
+      },
+      "disabled",
+   },
    {
       CORE_OPTION_NAME "_analog_stick_deadzone",
       "Analog Stick Deadzone",
